#include <stdio.h>
/* bprint ---
 *	print banners horizontally
 *	Options:
 *	[-w n]	the # of characters per line of output.
 *	[-]	read lines for banner from stdin 
 *	[-f]	read lines for banner from file 
 *	[-i]	invert the fill and space characters
 *	[-cc]	use c as the fill character
 *		default is to use the current character as fill
 *	[-bb]	use b as the background (space) character
 *	All other arguments are used as strings for output by bprint.
 *	Options & strings may be used any number of times in the argument list.
 *	If no banner files or strings are found stdin is used.
 *	If line width is too short only an error message to stderr is produced.
 *
 *$Log:	main.v $
 * Revision 1.4  83/05/07  11:13:38  dave
 * added defines in ifdefs so bprint & bprint1 can be selected from the
 * same library. The symbols are changed to make the 2 versions different.
 * 
 * Revision 1.3  83/04/11  14:19:18  herber
 * added inverse image option and changed calling sequence for bprint
 * 
 * Revision 1.2  83/04/07  16:22:59  herber
 * added -i (invert) option.
 * started on forground and background specification
 * 
 * Revision 1.1  83/04/06  15:53:54  herber
 * Initial revision
 * 
 */
#ifdef	BPRINT1
#define	bprint	bprint1
#endif	BPRINT1

main( argc, argv )
int	argc;
char	**argv;
{
	static char rcsid[] = "$Header:   main.v  Revision 1.4  83/05/07  11:13:38  dave  Exp$";
	int	linewidth;	/* # of characters in output line */
	int	anyout = 0;	/* any bprint output */
	FILE	*fp;		/* other input files */
	int	invert;		/* flag for inverted characters */
	char	bgchar = '\0';	/* background fill character */
	char	fgchar  = '\0';	/* forground fill character */

	linewidth=80;
	for ( argv++,argc--; argc > 0; argv++,argc-- )
	{
		switch ( **argv )
		{
		case '-':	/* option flag */
			switch ( (*argv)[1] )
			{
			case  '\0':
				file_bprint( stdin, fgchar, bgchar, invert, linewidth );
				anyout++;
				break;
			case 'i':
				invert++;
				break;
			case 'c':
				fgchar = (*argv)[2];
				break;
			case 'b':
				bgchar = (*argv)[2];
				break;
			case 'w':	/* width option */
				if ( --argc <= 0
					|| (linewidth = atoi(*(++argv))) <= 0 )
				{
					usage();
				}
				break;
			case 'f':	/* file argument option */
				if ( --argc <= 0
					|| (fp = fopen(*(++argv),"r")) == NULL)
				{
					fprintf( stderr, "file=%s\n", *argv );
					usage();
				}
				file_bprint( fp, fgchar, bgchar, invert, linewidth );
				fclose( fp );
				anyout++;
				break;
			default:
				usage();
				break;
			}
			break;

		default:	/* string to bprint */
			bprint( stdout, *argv, fgchar, bgchar, invert, linewidth );
			anyout++;
			break;
		}
	}

	if ( !anyout )	/* no banner strings found */
	{
		file_bprint( stdin, fgchar, bgchar, invert, linewidth );
	}
	exit(0);
}

file_bprint( fp, fgchar, bgchar, invert, linewidth )
FILE	*fp;			/* input file to bprint */
int	invert;			/* flag for inverted characters */
char	bgchar;			/* background fill character */
char	fgchar;			/* forground fill character */
int	linewidth;		/* # of characters in output line */
{
	char	linebuf[BUFSIZ];

	while ( fgets( linebuf, BUFSIZ, fp ) != NULL )
	{
		if ( bprint( stdout, linebuf, fgchar, bgchar, invert, linewidth ) == 1 )
			break;
	}
}

usage()
{
	fprintf( stderr,
	"usage: bprint [-bc] [-cc] [-i] [-w n] [-f filename] [-] string\n");
	exit(1);
}
