/*
 *			D D E V S T A T . C
 *
 *	This file handles the device status files for the daemon.
 */
#include <stdio.h>
#include "queue.h"
#include "devstat.h"
#include <sys/stat.h>

extern	struct device *devhead;
extern	lock_t Dev_lock;
extern	char *Admprefix;

#define	CHECK	1
#define	UPDATE	0

FILE	*dsopen();

devcheck()
{
	struct usrdevstat uds;
	struct stat sb;
	register struct device *dp;
	register FILE *fp;

	for( dp = devhead; dp != NULL; dp = dp->dv_nextdev ) {
		if(( fp = dsopen(dp, CHECK)) == NULL )
			continue;
		fseek(fp, (long) sizeof (struct daedevstat), 0);
		if(fread( &uds, sizeof uds, 1, fp ) != 1) {
			complain("bad devstat file for %s", dp->dv_dname);
			fclose(fp);
			continue;
		}
		fstat (fileno(fp), &sb);
		dp->dv_mtime = sb.st_mtime;
		fclose(fp);
		strncpy (dp->dv_forms, uds.uds_forms, FORMSIZ);
		dp->dv_status &= D_DAEBITS;
		dp->dv_status |= (uds.uds_status & ~D_DAEBITS);

		/*
		 *  If the user disables a device, clear the FAILED flag
		 */
		if((dp->dv_status & (D_DISABLED|D_FAILED))
		  == (D_DISABLED|D_FAILED)) {
			dp->dv_status &= ~D_FAILED;
		  	devactive ( dp, 0, "Idle" );
		}
	}
}

/*
 *	D E V A C T I V E (  )
 *
 *	Pid argument is either the pid of the new file control process,
 *	0 if there is now no active job on that device.
 */
devactive( dp, pid, msg )
register struct device *dp;
int	pid;
char	*msg;
{
	register FILE *fp;
	struct devstat ds;
	struct	stat sb;

	lock( &Dev_lock, 2 );	/* Try and lock out others */
	if(( fp = dsopen(dp, UPDATE)) == NULL ) {
		unlock( &Dev_lock );
		return;
	}
	if(fread( &ds, sizeof ds, 1, fp ) != 1) {
		complain("bad devstat file for %s", dp->dv_dname);
		fclose(fp);
		unlock( &Dev_lock );
		return;
	}

	dp->dv_status &= D_DAEBITS;
	dp->dv_status |= (ds.uds.uds_status & ~D_DAEBITS);
	strncpy (dp->dv_forms, ds.uds.uds_forms, FORMSIZ);
		
	/*
	 *  If the user disables a device, clear the FAILED flag
	 */
	if(dp->dv_status & D_DISABLED)
		dp->dv_status &= ~D_FAILED;
	ds.dds.dds_status &= ~D_DAEBITS;
	ds.dds.dds_status |= (dp->dv_status & D_DAEBITS);
	ds.dds.dds_pid = pid;

	if( pid )
		strncpy(ds.dds.dds_rname, dp->dv_actreq->r_name, QDIRSIZ);

	rewind( fp );
	if(fwrite( &ds.dds, sizeof ds.dds, 1, fp ) != 1)
		complain("bad devstat file for %s", dp->dv_dname);
	fseek (fp, (long)(sizeof (struct devstat)), 0);
	fwrite (msg, 1, strlen(msg)+1, fp);
	fflush(fp);
	fstat (fileno(fp), &sb);
	dp->dv_mtime = sb.st_mtime;
	fclose(fp);
	unlock( &Dev_lock );
}

FILE *
dsopen( dp, check )
register struct device *dp;
register int check;
{
	register FILE *fp;
	struct	stat sb;
	char	namebuf[PATHSIZ];

	sprintf( namebuf, "%s/D_%s", Admprefix, dp->dv_dname );
	if( check ) {
		if(stat(namebuf, &sb) < 0) {
			complain("Can't stat devstat file for %s (%s)",
				  namebuf, sys_errlist[errno] );
			return( NULL );
		}
		if( sb.st_mtime <= dp->dv_mtime )
			return( NULL );
	}

	if(( fp = fopen( namebuf, "r+" )) == NULL )
		complain( "Can't open devstat file %s (%s)",
			  namebuf, sys_errlist[errno] );
	return( fp );
}

devsinit(dp)
register struct device *dp;
{
	register int fd;
	struct devstat ds;
	static namebuf[PATHSIZ];

	sprintf( namebuf, "%s/D_%s", Admprefix, dp->dv_dname );
	lock( &Dev_lock, NOWAIT );

	if (access(namebuf, RACCESS|WACCESS) != 0) {
		if ((fd = creat(namebuf, 0666)) < 0) {
			complain("Can't create devstat file %s (%s)",
				 namebuf, sys_errlist[errno]);
			return (NULL);
		}
		strncpy(ds.uds.uds_forms, "*Empty*", FORMSIZ);
		ds.uds.uds_status = 0;
		ds.dds.dds_status = dp->dv_status & D_DAEBITS;
		ds.dds.dds_pid = 0;
		if(write(fd, &ds, sizeof ds) != sizeof ds)
			complain("Can't initialize devstat file %s (%s)",
				 namebuf, sys_errlist[errno]);
		close (fd);
	}
	chmod (namebuf, 0666);
	devactive(dp, 0, "Idle");	/* Does an unlock of Dev_lock */
}

openstat( dp, desired )
register struct device *dp;
register int desired;
{
	int	fd;
	char	namebuf[PATHSIZ];

	sprintf( namebuf, "%s/D_%s", Admprefix, dp->dv_dname );
	if(( fd = open( namebuf, 2 )) < 0 ) {
		complain( "Can't open devstat file %s (%s)",
			  namebuf, sys_errlist[errno] );
		fd = open ("/dev/null", 2);
	}
	dup2(fd, desired);
}
