#
/*
 *	    M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *
 *				  D M E M . C
 *
 *			   Douglas P. Kingston III
 *
 *		Ballistics Research Laboratory,  March 1982
 *
 *	________________________________________________________________
 *
 *				Revision History
 *				================
 *	30 Mar 83  DPK	Modified zapque and zapdev to return the next
 *			structure in the chain.
 */

#include <stdio.h>
#include "queue.h"
#include <signal.h>

extern struct request *delhead;
extern int delayupdate;
extern char *Admprefix;
char	*xmalloc();
char	*malloc();

/*
 *			X M A L L O C
 *	Xmalloc behaves like malloc except that it is always successful.
 *	If sufficent memory does not exist, it will look for things to
 *	throw away to reclaim enough space.
 */
char *
xmalloc ( size )
int size;
{
	register struct request *rp;
	char *mp;

	while (( mp = malloc (size)) == NULL ) {
		/*
		 *  Bad news, start looking for more easy core
		 */
		complain ("Xmalloc trying...");

		rp = delhead;
		if (rp != NULL) {
			/* Burn the head of the delayed request list */
			delayupdate = 1;
			delhead = rp->r_nextreq;
			renew (rp->r_name);
			free (rp);
			continue;
		}

		bomb ("Xmalloc failed to succeed");
	}

	return (mp);
}

char *
strdup ( strp )
register char	*strp;
{
	register char *cp;

	cp = xmalloc ( strlen (strp) + 1 );
	strcpy (cp, strp);
	return (cp);
}

struct queue *
zapque( qp )
register struct queue *qp;
{
	char	qfname[PATHSIZ];
	struct queue *nextqp;
	register struct request *rp;

	sprintf( qfname, "%s/Q_%s", Admprefix, qp->q_name );
	unlink( qfname );
	free( qp->q_name );
	for( rp = qp->q_readyreq; rp != NULL; rp = rp->r_nextreq ) {
		zapreq( rp );
		hold( rp->r_name );
		free( rp );
	}
	nextqp = qp->q_nextq;
	free( qp );
	return( nextqp );
}

struct device *
zapdev( dp )
register struct device *dp;
{
	char	dfname[PATHSIZ];
	register struct request *rp;
	struct device *nextdp;

	sprintf( dfname, "%s/D_%s", Admprefix, dp->dv_dname );
	unlink( dfname );
	if( dp->dv_pid > 1 )		/* Paranoid */
#if V6
		kill( dp->dv_pid, SIGTERM );
#endif
#if BSD42 | BSD41 | BSD41c | BRLUNIX
		killpg( dp->dv_pid, SIGTERM );
#endif
#if SYS3 | V7
		kill( -dp->dv_pid, SIGTERM );
#endif
	if( rp = dp->dv_actreq ) {
		if ( queue( rp ) < 0 ) {
			complain("enroll of %s failed -- no queue",
				rp->r_name );
			zapreq( rp );
			hold( rp->r_name );
			free( rp );
		}
	}
	free( dp->dv_dname );
	free( dp->dv_device );
	nextdp = dp->dv_nextdev;
	free( dp );
	return( nextdp );
}
