/*
 *	    M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *		     TIG Plot Server Process  (VTCSERVER)
 *
 *	Note that a pipe to the Daemon will be on file descriptor 3.
 *	________________________________________________________________
 *
 *				Revision History
 *				================
 */

#include <stdio.h>
#include <signal.h>
#include "queue.h"
#include "qreplys.h"

struct	request r;
int	lpoptions;

struct	request *gethdr();
char	*index();
int	doflush();
int	dorestart();
int	vtc();				/* rasterizer */

extern	char *Sitefile;
extern	char *Qdatadir;

/*
 *		M A I N  ( )
 *  As a MDQS queue server, this process will be started up
 *  with the queue control file as the standard input, and
 *  the device of interest on the standard output.
 */
main( argc, argv )
int	argc;
char	**argv;
{
	char	datafile[LINESIZ];
	char	linebuf[LINESIZ];
	char	*cp;
	
	int 	val;
	
	long     usedlines;
	long     linelimit;
	
	signal( SIGHUP, SIG_IGN );		/* Paranoid. */
	signal( SIGEMT, dorestart );
	signal( SIGTERM, doflush );

	setbuf( stdout, NULL );		/* vtc() does its own */

	usedlines = 0L;
	linelimit = 5000L;
	if (gethdr( stdin, &r ) == (struct request *) NULL)
	{
		err( "Error reading request" );
		retmsg (RP_FATAL, "Error reading request header");
		exit( 10 );
	}
	while (fgets( linebuf, sizeof linebuf, stdin ) != NULL)
	{
		if ((cp = index( linebuf, '\n' )) != NULL)
			*cp = '\0';
		switch (linebuf[0]) 
		{
		case 'D':
			if (chdir( &linebuf[1] ) < 0)
			{
				err( "Error trying chdir (%s,%s)",
				&linebuf[1], sys_errlist[errno] );
 				chdir( "/tmp" );  /* A safe default */
			}
			break;

		case 'O':
			setoptions( &linebuf[1] );
			break;

		case 'I':
			sprintf( datafile, "%s/%s", Qdatadir,
							&linebuf[1] );
			if ((linelimit -= usedlines) > 0L)
			{
				val = vtc( r.r_copies,datafile,
					       linelimit,&usedlines );
				unlink( datafile );
				if (val != 0)	/* linelimit exceeded */
				{
					retmsg (RP_FLUSH, "Line limit exceeded");
					exit( 0 );
				}
			}
			break;

		case 'F':
			if ((linelimit -= usedlines) > 0L)
			{
				val = vtc( r.r_copies,&linebuf[1],
					       linelimit,&usedlines );
				if (val != 0)	/* linelimit exceeded */
				{
					retmsg (RP_FLUSH, "Line limit exceeded");
					exit( 0 );
				}
			}
			if (lpoptions & NOKEEP)
				unlink( &linebuf[1] );
			break;

		case 'U':
		case 'B':
		case 'H':
		case 'T':
		default:
			break;
		}
	}
	retmsg (RP_OK, 0);
	exit( 0 );
}

/*
 *		S E T O P T I O N S  ( )
 *  Setoptions is passed the ascii string numerical representation
 *  of the lpoptions flags word.  The bit definitions are in queue.h.
 *  The string is a decimal representation.
 */
setoptions( optstr )
char *optstr;
{
	lpoptions = atoi( optstr );
}
	
/*
 *		E R R ( )
 *	Called by banner() and its subfunctions, amongst others
 */
/* VARARGS */
err( fmt, a, b, c, d )
char *fmt;
{
	fputs( "PLTSERVER: ", stderr );
	fprintf( stderr, fmt, a, b, c, d );
	putc( '\n', stderr );
}
